//
//  UHRTestBenchTests.m
//  TestBenchTests
//
//  Created by Elisa Silva on 07/10/21.
//
//  Copyright 2021 Elisa Silva
//  
//  Permission is hereby granted, free of charge, to any person obtaining a 
//  copy of this software and associated documentation files (the "Software"), 
//  to deal in the Software without restriction, including without limitation 
//  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//  and/or sell copies of the Software, and to permit persons to whom the Software 
//  is furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in 
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
//  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
//  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#import <XCTest/XCTest.h>
#import <OCMock/OCMock.h>

#import "UHRTestBench.h"
#import "UHRTestBenchScript.h"

@interface UHRTestBenchTests : XCTestCase

@property UHRTestBench *bench;
@property id mockedScript;

@end

@implementation UHRTestBenchTests

- (void)setUp {
    _mockedScript = OCMClassMock(UHRTestBenchScript.class);
    _bench = [[UHRTestBench alloc] initWithModule:nil withScript:_mockedScript];
}

- (void)testInitOfTestBench {
    XCTAssertNotNil(_bench);
    OCMVerify([_mockedScript useXCTestIntegration:YES]);
}

- (void)testCallScriptMethodsDuringRun {
    OCMStub([_mockedScript passScriptForModule:nil atTime:9]).andReturn(YES);
    XCTAssertTrue([_bench runTestBenchUpToTime:10]);
    
    OCMVerify([_mockedScript applyOnRiseChangesToModule:nil atTime:5]);
    OCMVerify([_mockedScript applyOnFallChangesToModule:nil atTime:5]);
    OCMVerify([_mockedScript checkOnHighContrainsForModule:nil atTime:5]);
    OCMVerify([_mockedScript checkOnLowContrainsForModule:nil atTime:5]);
    OCMVerify([_mockedScript callCallbackWithModule:nil atTime:5]);
}

- (void)testTimeoutRun {
    XCTAssertFalse([_bench runTestBenchUpToTime:10]);
}

@end
